orangefs_prepare_debugfs_help_string() constructs the debugfs help text in
a dynamically allocated buffer using repeated strlcat() calls across
keyword loops, rescanning the buffer on every append.

In preparation for removing the deprecated strlcat() API[1], use struct
seq_buf to build the help string. This tracks the write position across
the loops, allows formatting each keyword line with a single
seq_buf_printf() call, and checks for truncation via
seq_buf_has_overflowed().

Additionally, replace the final strlcat() call that copies the completed
string into debug_help_string with strscpy().

Link: https://github.com/KSPP/linux/issues/370 [1]
Cc: [email protected]
Assisted-by: Gemini:gemini-2.0-flash-exp [shell_command_run, editor_update_file]
Signed-off-by: Bill Wendling <[email protected]>
---
Cc: Russell King <[email protected]>
Cc: Huacai Chen <[email protected]>
Cc: WANG Xuerui <[email protected]>
Cc: Thomas Bogendoerfer <[email protected]>
Cc: "James E.J. Bottomley" <[email protected]>
Cc: Helge Deller <[email protected]>
Cc: Thomas Gleixner <[email protected]>
Cc: Ingo Molnar <[email protected]>
Cc: Borislav Petkov <[email protected]>
Cc: Dave Hansen <[email protected]>
Cc: [email protected]
Cc: "H. Peter Anvin" <[email protected]>
Cc: Ian Abbott <[email protected]>
Cc: H Hartley Sweeten <[email protected]>
Cc: Tony Luck <[email protected]>
Cc: Maarten Lankhorst <[email protected]>
Cc: Maxime Ripard <[email protected]>
Cc: Thomas Zimmermann <[email protected]>
Cc: David Airlie <[email protected]>
Cc: Simona Vetter <[email protected]>
Cc: Matthew Brost <[email protected]>
Cc: "Thomas Hellström" <[email protected]>
Cc: Rodrigo Vivi <[email protected]>
Cc: Dmitry Torokhov <[email protected]>
Cc: Matthias Schwarzott <[email protected]>
Cc: Mauro Carvalho Chehab <[email protected]>
Cc: Tony Nguyen <[email protected]>
Cc: Przemek Kitszel <[email protected]>
Cc: Andrew Lunn <[email protected]>
Cc: "David S. Miller" <[email protected]>
Cc: Eric Dumazet <[email protected]>
Cc: Jakub Kicinski <[email protected]>
Cc: Paolo Abeni <[email protected]>
Cc: Arend van Spriel <[email protected]>
Cc: Rob Herring <[email protected]>
Cc: Saravana Kannan <[email protected]>
Cc: Krzysztof Kozlowski <[email protected]>
Cc: Sylwester Nawrocki <[email protected]>
Cc: Peter Griffin <[email protected]>
Cc: Alim Akhtar <[email protected]>
Cc: Linus Walleij <[email protected]>
Cc: Anil Gurumurthy <[email protected]>
Cc: Sudarsana Kalluru <[email protected]>
Cc: "Martin K. Petersen" <[email protected]>
Cc: Trond Myklebust <[email protected]>
Cc: Anna Schumaker <[email protected]>
Cc: Mike Marshall <[email protected]>
Cc: Martin Brandenburg <[email protected]>
Cc: Kees Cook <[email protected]>
Cc: Jiri Pirko <[email protected]>
Cc: Simon Horman <[email protected]>
Cc: Chuck Lever <[email protected]>
Cc: Jeff Layton <[email protected]>
Cc: NeilBrown <[email protected]>
Cc: Olga Kornievskaia <[email protected]>
Cc: Dai Ngo <[email protected]>
Cc: Tom Talpey <[email protected]>
Cc: Jaroslav Kysela <[email protected]>
Cc: Takashi Iwai <[email protected]>
Cc: Bill Wendling <[email protected]>
Cc: Andrew Morton <[email protected]>
Cc: "Mike Rapoport (Microsoft)" <[email protected]>
Cc: Kanglong Wang <[email protected]>
Cc: Tiezhu Yang <[email protected]>
Cc: Qiang Ma <[email protected]>
Cc: Randy Dunlap <[email protected]>
Cc: Pengpeng Hou <[email protected]>
Cc: Ard Biesheuvel <[email protected]>
Cc: Breno Leitao <[email protected]>
Cc: Thorsten Blum <[email protected]>
Cc: Harshit Mogalapalli <[email protected]>
Cc: Greg Kroah-Hartman <[email protected]>
Cc: Lyude Paul <[email protected]>
Cc: Ashutosh Desai <[email protected]>
Cc: Imre Deak <[email protected]>
Cc: Dmitry Baryshkov <[email protected]>
Cc: Johan Hovold <[email protected]>
Cc: Johannes Berg <[email protected]>
Cc: Miri Korenblit <[email protected]>
Cc: Alexander Stein <[email protected]>
Cc: Cryolitia PukNgae <[email protected]>
Cc: Jiaming Zhang <[email protected]>
Cc: Will Porter <[email protected]>
Cc: Cen Zhang <[email protected]>
Cc: "Cássio Gabriel" <[email protected]>
Cc: Rong Zhang <[email protected]>
Cc: Arun Raghavan <[email protected]>
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
Cc: [email protected]
---
 fs/orangefs/orangefs-debugfs.c | 36 +++++++++++++---------------------
 1 file changed, 14 insertions(+), 22 deletions(-)

diff --git a/fs/orangefs/orangefs-debugfs.c b/fs/orangefs/orangefs-debugfs.c
index 9f94919a6bc6..6e2f9887eab4 100644
--- a/fs/orangefs/orangefs-debugfs.c
+++ b/fs/orangefs/orangefs-debugfs.c
@@ -37,6 +37,7 @@
  */
 #include <linux/debugfs.h>
 #include <linux/slab.h>
+#include <linux/seq_buf.h>
 
 #include <linux/uaccess.h>
 
@@ -623,10 +624,10 @@ int orangefs_prepare_debugfs_help_string(int at_boot)
        char *client_title = "Client Debug Keywords:\n";
        char *kernel_title = "Kernel Debug Keywords:\n";
        size_t string_size =  DEBUG_HELP_STRING_SIZE;
-       size_t result_size;
        size_t i;
        char *new;
        int rc = -EINVAL;
+       struct seq_buf s;
 
        gossip_debug(GOSSIP_UTILS_DEBUG, "%s: start\n", __func__);
 
@@ -640,17 +641,14 @@ int orangefs_prepare_debugfs_help_string(int at_boot)
                goto out;
        }
 
+       seq_buf_init(&s, new, string_size);
+
        /*
-        * strlcat(dst, src, size) will append at most
-        * "size - strlen(dst) - 1" bytes of src onto dst,
-        * null terminating the result, and return the total
-        * length of the string it tried to create.
-        *
         * We'll just plow through here building our new debug
         * help string and let strlcat take care of assuring that
         * dst doesn't overflow.
         */
-       strlcat(new, client_title, string_size);
+       seq_buf_puts(&s, client_title);
 
        if (!at_boot) {
 
@@ -665,24 +663,18 @@ int orangefs_prepare_debugfs_help_string(int at_boot)
                        goto out;
                }
 
-               for (i = 0; i < cdm_element_count; i++) {
-                       strlcat(new, "\t", string_size);
-                       strlcat(new, cdm_array[i].keyword, string_size);
-                       strlcat(new, "\n", string_size);
-               }
+               for (i = 0; i < cdm_element_count; i++)
+                       seq_buf_printf(&s, "\t%s\n", cdm_array[i].keyword);
        }
 
-       strlcat(new, "\n", string_size);
-       strlcat(new, kernel_title, string_size);
+       seq_buf_puts(&s, "\n");
+       seq_buf_puts(&s, kernel_title);
 
-       for (i = 0; i < num_kmod_keyword_mask_map; i++) {
-               strlcat(new, "\t", string_size);
-               strlcat(new, s_kmod_keyword_mask_map[i].keyword, string_size);
-               result_size = strlcat(new, "\n", string_size);
-       }
+       for (i = 0; i < num_kmod_keyword_mask_map; i++)
+               seq_buf_printf(&s, "\t%s\n", 
s_kmod_keyword_mask_map[i].keyword);
 
        /* See if we tried to put too many bytes into "new"... */
-       if (result_size >= string_size) {
+       if (seq_buf_has_overflowed(&s)) {
                kfree(new);
                goto out;
        }
@@ -692,7 +684,7 @@ int orangefs_prepare_debugfs_help_string(int at_boot)
        } else {
                mutex_lock(&orangefs_help_file_lock);
                memset(debug_help_string, 0, DEBUG_HELP_STRING_SIZE);
-               strlcat(debug_help_string, new, string_size);
+               strscpy(debug_help_string, new, DEBUG_HELP_STRING_SIZE);
                mutex_unlock(&orangefs_help_file_lock);
                kfree(new);
        }
@@ -640,17 +641,14 @@ int orangefs_prepare_debugfs_help_string(int at_boot)
         * We'll just plow through here building our new debug
-        * help string and let strlcat take care of assuring that
+        * help string and let seq_buf take care of assuring that
         * dst doesn't overflow.
         */
-- 
2.55.0.1032.g73a4cd73de-goog

Reply via email to